//
// matrix.cc
// jason Kinser
// 2 Feb 1995
// 15 October 1995
// 15 April 1996  Added == .
//

#include "matrix.h"
#include <iostream>
#include <fstream>

using std::cout;
using std::endl;

#define foralli for(int i=0; i<vert; ++i)
#define forallj for(int j=0; j<horz; ++j)

typedef unsigned char byte;

// ********* Begin matrix class

matrix::matrix( int d1, int d2 )
    : vert(d1), horz(d2),
      data(d1 * d2, 0)
{}

matrix::matrix( const matrix & A )
    : vert(A.vert), horz(A.horz),
      data(A.data.begin(), A.data.end())
{}

matrix::~matrix()
{}

matrix& matrix::operator=( const matrix & b)
{
    copy(b.data.begin(), b.data.end(), data.begin());
    return *this;
}

float matrix::Max() const
{
    std::vector<float>::const_iterator i = data.begin();
    float max = *i;
    while(++i != data.end())
	if(*i > max) max = *i;
    
    return max;
}

float matrix::Min() const
{
    std::vector<float>::const_iterator i = data.begin();
    float min = *i;
    while(++i != data.end())
	if(*i < min) min = *i;
    
    return min;
}

void matrix::Clear(const float A)
{
    for(std::vector<float>::iterator i = data.begin(); i != data.end(); ++i)
	*i = A;
}

matrix& matrix::operator *= (const float f)
{
    for(std::vector<float>::iterator i = data.begin(); i != data.end(); ++i)
	*i *= f;
    return *this;
}

matrix& matrix::operator += (const float f)
{
    for(std::vector<float>::iterator i = data.begin(); i != data.end(); ++i)
	*i += f;
    return *this;
}

matrix& matrix::operator *= (const matrix& m) {
    std::vector<float>::const_iterator b = m.data.begin();
    for(std::vector<float>::iterator a=data.begin(); a != data.end();
	++a, ++b)
	*a *= *b;
    return *this;
}

matrix& matrix::operator += (const matrix& m) {
    std::vector<float>::const_iterator b = m.data.begin();
    for(std::vector<float>::iterator a=data.begin(); a != data.end();
	++a, ++b)
	*a += *b;
    return *this;
}

float matrix::Sum( void ) const
{
    float sum = 0.0;
    for(std::vector<float>::const_iterator i = data.begin(); i != data.end();
	++i)
	sum += *i;
    
    return sum;
}

void matrix::Load (char *name)
{
    std::ifstream in(name);
    Load(in);
}

void matrix::Save (char *name)
{
    std::ofstream out(name);
    Save(out);
}

void matrix::Load(std::istream& in)
{
    for( int i=0; i<vert; i++ ) for( int j=0; j<horz; j++ )
	in.read((char*) &(*this)(i, j), sizeof(float));
}

void matrix::Save(std::ostream& out)
{
    for( int i=0; i<vert; i++ )  for( int j=0; j<horz; j++ )
	out.write((char*) &(*this)(i, j), sizeof(float));
}

// LoadByte
void matrix::LoadByte( char * name, int header )
{
    std::ifstream in(name);
    if(!in) {
	cout << "Cannot open " << name << endl;
	exit(1); }
    
    char * head = new char[header];
    in.read(head, header);
    delete head;
    
    byte ** A = new byte * [vert];
    foralli A[i] = new byte[ horz ];
    
    foralli in.read((char*)A[i], horz);
    
    foralli forallj {
	float t = A[i][j];
	(*this)(i, j) = t;
    }
    
    foralli delete A[i];
    delete A;
}
